@@ -70,7 +70,7 @@
 #define yychar          i386_char
 
 /* Copy the first part of user declarations.  */
-#line 1 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:339  */
+#line 1 "i386_parse.y" /* yacc.c:339  */
 
 /* Parser for i386 CPU description.
    Copyright (C) 2004, 2005, 2007, 2008, 2009 Red Hat, Inc.
@@ -349,7 +349,7 @@ extern int i386_debug;
 
 union YYSTYPE
 {
-#line 216 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:355  */
+#line 216 "i386_parse.y" /* yacc.c:355  */
 
   unsigned long int num;
   char *str;
@@ -1473,7 +1473,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 247 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 247 "i386_parse.y" /* yacc.c:1646  */
     {
 		      if (error_message_count != 0)
 			error (EXIT_FAILURE, 0,
@@ -1485,25 +1485,25 @@ yyreduce:
     break;
 
   case 5:
-#line 261 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 261 "i386_parse.y" /* yacc.c:1646  */
     { new_bitfield ((yyvsp[-1].str), (yyvsp[0].num)); }
 #line 1491 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 263 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 263 "i386_parse.y" /* yacc.c:1646  */
     { new_bitfield ((yyvsp[0].str), -1); }
 #line 1497 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 7:
-#line 265 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 265 "i386_parse.y" /* yacc.c:1646  */
     { new_bitfield ((yyvsp[0].str), -2); }
 #line 1503 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 8:
-#line 267 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 267 "i386_parse.y" /* yacc.c:1646  */
     {
 		      struct synonym *newp = xmalloc (sizeof (*newp));
 		      newp->from = (yyvsp[-1].str);
@@ -1519,7 +1519,7 @@ yyreduce:
     break;
 
   case 12:
-#line 286 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 286 "i386_parse.y" /* yacc.c:1646  */
     {
 		      if ((yyvsp[-3].field) != NULL && strcmp ((yyvsp[-3].field)->name, "RE") != 0
 			  && strcmp ((yyvsp[-3].field)->name, "R") != 0)
@@ -1606,7 +1606,7 @@ yyreduce:
     break;
 
   case 14:
-#line 372 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 372 "i386_parse.y" /* yacc.c:1646  */
     {
 		      struct known_bitfield search;
 		      search.name = (yyvsp[0].str);
@@ -1625,13 +1625,13 @@ yyreduce:
     break;
 
   case 15:
-#line 387 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 387 "i386_parse.y" /* yacc.c:1646  */
     { (yyval.field) = NULL; }
 #line 1631 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 391 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 391 "i386_parse.y" /* yacc.c:1646  */
     {
 		      check_bits ((yyvsp[0].bit));
 
@@ -1645,7 +1645,7 @@ yyreduce:
     break;
 
   case 17:
-#line 401 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 401 "i386_parse.y" /* yacc.c:1646  */
     {
 		      check_bits ((yyvsp[0].bit));
 		      (yyval.bit) = (yyvsp[0].bit);
@@ -1654,7 +1654,7 @@ yyreduce:
     break;
 
   case 18:
-#line 408 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 408 "i386_parse.y" /* yacc.c:1646  */
     {
 		      struct bitvalue *runp = (yyvsp[-1].bit);
 		      while (runp->next != NULL)
@@ -1666,13 +1666,13 @@ yyreduce:
     break;
 
   case 19:
-#line 416 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 416 "i386_parse.y" /* yacc.c:1646  */
     { (yyval.bit) = (yyvsp[0].bit); }
 #line 1672 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 420 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 420 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1683,7 +1683,7 @@ yyreduce:
     break;
 
   case 21:
-#line 427 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 427 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1694,7 +1694,7 @@ yyreduce:
     break;
 
   case 22:
-#line 434 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 434 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      struct known_bitfield search;
@@ -1718,19 +1718,19 @@ yyreduce:
     break;
 
   case 23:
-#line 456 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 456 "i386_parse.y" /* yacc.c:1646  */
     { (yyval.arg) = (yyvsp[0].arg); }
 #line 1724 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 24:
-#line 458 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 458 "i386_parse.y" /* yacc.c:1646  */
     { (yyval.arg) = NULL; }
 #line 1730 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 462 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 462 "i386_parse.y" /* yacc.c:1646  */
     {
 		      struct argument *runp = (yyvsp[-2].arg);
 		      while (runp->next != NULL)
@@ -1744,7 +1744,7 @@ yyreduce:
     break;
 
   case 26:
-#line 472 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 472 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.arg) = xmalloc (sizeof (struct argument));
 		      (yyval.arg)->name = combine ((yyvsp[0].name));
@@ -1754,7 +1754,7 @@ yyreduce:
     break;
 
   case 27:
-#line 480 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 480 "i386_parse.y" /* yacc.c:1646  */
     {
 		      struct argname *runp = (yyvsp[-1].name);
 		      while (runp->next != NULL)
@@ -1766,13 +1766,13 @@ yyreduce:
     break;
 
   case 28:
-#line 488 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 488 "i386_parse.y" /* yacc.c:1646  */
     { (yyval.name) = (yyvsp[0].name); }
 #line 1772 "i386_parse.c" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 491 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 491 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = nfield;
@@ -1808,7 +1808,7 @@ yyreduce:
     break;
 
   case 30:
-#line 523 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 523 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1821,7 +1821,7 @@ yyreduce:
     break;
 
   case 31:
-#line 532 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 532 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1832,7 +1832,7 @@ yyreduce:
     break;
 
   case 32:
-#line 539 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1646  */
+#line 539 "i386_parse.y" /* yacc.c:1646  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -2073,7 +2073,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 549 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1906  */
+#line 549 "i386_parse.y" /* yacc.c:1906  */
 
 
 static void
