@@ -153,6 +153,7 @@ read_srclines (Dwarf *dbg,
 {
   int res = -1;
 
+  struct filelist *filelist = NULL;
   size_t nfilelist = 0;
   size_t ndirlist = 0;
 
@@ -445,7 +446,7 @@ read_srclines (Dwarf *dbg,
       },
       .next = NULL
     };
-  struct filelist *filelist = &null_file;
+  filelist = &null_file;
   nfilelist = 1;
 
   /* Allocate memory for a new file.  For the first MAX_STACK_FILES
@@ -507,11 +508,10 @@ read_srclines (Dwarf *dbg,
 		{
 		  /* This value could be NULL in case the DW_AT_comp_dir
 		     was not present.  We cannot do much in this case.
-		     The easiest thing is to convert the path in an
-		     absolute path.  */
+		     Just keep the file relative.  */
 		  cp = stpcpy (cp, dirarray[diridx].dir);
+		  *cp++ = '/';
 		}
-	      *cp++ = '/';
 	      strcpy (cp, fname);
 	      assert (strlen (new_file->info.name)
 		      < dirarray[diridx].len + 1 + fnamelen + 1);
@@ -802,11 +802,12 @@ read_srclines (Dwarf *dbg,
 		    if (dirarray[diridx].dir != NULL)
 		      /* This value could be NULL in case the
 			 DW_AT_comp_dir was not present.  We
-			 cannot do much in this case.  The easiest
-			 thing is to convert the path in an
-			 absolute path.  */
-		      cp = stpcpy (cp, dirarray[diridx].dir);
-		    *cp++ = '/';
+			 cannot do much in this case.  Just
+			 keep the file relative.  */
+		      {
+			cp = stpcpy (cp, dirarray[diridx].dir);
+			*cp++ = '/';
+		      }
 		    strcpy (cp, fname);
 		  }
 
