@@ -36,8 +36,7 @@
 
 
 const char *
-ebl_backend_name (ebl)
-     Ebl *ebl;
+ebl_backend_name (Ebl *ebl)
 {
   return ebl != NULL ? ebl->emulation : gettext ("No backend");
 }
