@@ -1,5 +1,5 @@
 /* Return converted data from raw chunk of ELF file.
-   Copyright (C) 2007, 2014 Red Hat, Inc.
+   Copyright (C) 2007, 2014, 2015 Red Hat, Inc.
    This file is part of elfutils.
 
    This file is free software; you can redistribute it and/or modify
@@ -41,11 +41,7 @@
 #include "common.h"
 
 Elf_Data *
-elf_getdata_rawchunk (elf, offset, size, type)
-     Elf *elf;
-     off64_t offset;
-     size_t size;
-     Elf_Type type;
+elf_getdata_rawchunk (Elf *elf, off_t offset, size_t size, Elf_Type type)
 {
   if (unlikely (elf == NULL))
     return NULL;
@@ -84,8 +80,7 @@ elf_getdata_rawchunk (elf, offset, size, type)
     {
     /* If the file is mmap'ed we can use it directly, if aligned for type.  */
       char *rawdata = elf->map_address + elf->start_offset + offset;
-      if (ALLOW_UNALIGNED ||
-	  ((uintptr_t) rawdata & (align - 1)) == 0)
+      if (((uintptr_t) rawdata & (align - 1)) == 0)
 	rawchunk = rawdata;
       else
 	{
